use 5.012;
use warnings;

use Test::More;
BEGIN { use_ok('Panda::Time') };
BEGIN { use_ok('Panda::Date') };
BEGIN { use_ok('Panda::Date::Rel') };
BEGIN { use_ok('Panda::Date::Int') };

done_testing();
